@@ -91,7 +91,9 @@ namespace Tomboy
 
 		void ShowHelpVerb ()
 		{
-			Tomboy.ActionManager ["ShowHelpAction"].Activate ();
+			// Don't use the ActionManager in this case because
+			// the handler won't know about the Screen.
+			GuiUtils.ShowHelp ("tomboy.xml", null, Screen, null);
 		}
 
 		void ShowAboutVerb ()
