@@ -35,6 +35,7 @@ namespace Tomboy
 			list.Add (Gdk.Atom.Intern ("_NETSCAPE_URL", false), 0, 1);
 			
 			KeyPressEvent += KeyPressed;
+			ButtonPressEvent += ButtonPressed;
 		}
 
 		public static int DefaultMargin
@@ -163,6 +164,12 @@ namespace Tomboy
 			
 			args.RetVal = ret_value;
 		}
+		
+		[GLib.ConnectBefore()]
+		void ButtonPressed (object sender, Gtk.ButtonPressEventArgs args)
+		{
+			((NoteBuffer) Buffer).CheckSelection ();
+		}
 	}
 
 	public class NoteWindow : ForcedPresentWindow 
