@@ -120,6 +120,8 @@ namespace Tomboy
 
 	public abstract class AbstractPlugin : IPlugin
 	{
+		bool disposing = false;
+
 		~AbstractPlugin ()
 		{
 			Dispose (false);
@@ -127,13 +129,20 @@ namespace Tomboy
 
 		public void Dispose ()
 		{
+			disposing = true;
 			Dispose (true);
+
 			GC.SuppressFinalize (this);
 		}
 
 		protected virtual void Dispose (bool disposing)
 		{
 		}
+
+		public bool IsDisposing
+		{
+			get { return disposing; }
+		}
 	}
 
 	public abstract class NotePlugin : AbstractPlugin
@@ -182,14 +191,36 @@ namespace Tomboy
 			get { return note; }
 		}
 
+		public bool HasBuffer
+		{
+			get { return note.HasBuffer; }
+		}
+
 		public NoteBuffer Buffer
 		{
-			get { return note.Buffer; }
+			get
+			{
+				if (IsDisposing && !HasBuffer)
+					throw new InvalidOperationException ("Plugin is disposing already");
+
+				return note.Buffer; 
+			}
+		}
+
+		public bool HasWindow
+		{
+			get { return note.HasWindow; }
 		}
 
 		public NoteWindow Window
 		{
-			get { return note.Window; }
+			get
+			{
+				if (IsDisposing && !HasWindow)
+					throw new InvalidOperationException ("Plugin is disposing already");
+
+				return note.Window; 
+			}
 		}
 
 		public NoteManager Manager
@@ -222,6 +253,9 @@ namespace Tomboy
 
 		public void AddPluginMenuItem (Gtk.MenuItem item)
 		{
+			if (IsDisposing)
+				throw new InvalidOperationException ("Plugin is disposing already");
+
 			if (plugin_menu_items == null)
 				plugin_menu_items = new List<Gtk.MenuItem> ();
 
@@ -233,6 +267,9 @@ namespace Tomboy
 
 		public void AddTextMenuItem (Gtk.MenuItem item)
 		{
+			if (IsDisposing)
+				throw new InvalidOperationException ("Plugin is disposing already");
+
 			if (text_menu_items == null)
 				text_menu_items = new List<Gtk.MenuItem> ();
 
@@ -333,7 +370,7 @@ namespace Tomboy
 
 			try {
 				Process.Start (command, args);
-			} catch (SystemException e) {
+			} catch (Exception e) {
 				Logger.Log ("Error opening file browser \"{0}\" to \"{1}\": {2}",
 					    command,
 					    plugins_dir,
@@ -343,7 +380,8 @@ namespace Tomboy
 
 		public void LoadPluginsForNote (Note note)
 		{
-			foreach (Type type in plugin_types) {
+			foreach (Type type in plugin_types as 
+					System.Collections.Generic.IEnumerable<Type>) {
 				if (IsPluginEnabled (type))
 					AttachPlugin (type, note);
 			}
@@ -404,6 +442,24 @@ namespace Tomboy
 			// Copy Plugins/DefaultPlugins.desktop file from resource
 			string default_desktop = "DefaultPlugins.desktop";
 			string default_path = Path.Combine (plugins_dir, default_desktop);
+			if (File.Exists (default_path)) {
+				// Check the existing file to make sure it has the correct path.
+				// If it doesn't, delete it so a correct one will installed.
+				StreamReader reader = null;
+				try {
+					reader = new StreamReader (File.OpenRead (default_path));
+					string contents = reader.ReadToEnd ();
+					if (contents.IndexOf ("file://" + Defines.SYS_PLUGINS_DIR) < 0)
+						File.Delete (default_path);
+				} catch (Exception e) {
+					Logger.Warn ("Could not update DefaultPlugins.desktop file: {0}",
+						e.Message);
+				} finally {
+					if (reader != null)
+						reader.Close ();
+				}
+			}
+
 			if (!File.Exists (default_path)) {
 				Assembly asm = Assembly.GetExecutingAssembly();
 				Stream stream = asm.GetManifestResourceStream (default_desktop);
@@ -431,7 +487,8 @@ namespace Tomboy
 			PluginTable note_plugins = null;
 			
 			if (attached_plugins.TryGetValue (deleted, out note_plugins)) {
-				foreach (NotePlugin plugin in note_plugins.Values) {
+				foreach (NotePlugin plugin in note_plugins.Values as
+						System.Collections.Generic.IEnumerable<NotePlugin>) {
 					try {
 						plugin.Dispose ();
 					} catch (Exception e) {
@@ -476,7 +533,8 @@ namespace Tomboy
 		void AttachPlugin (Type type)
 		{
 			if (typeof (NotePlugin).IsAssignableFrom (type)) {
-				foreach (Note note in attached_plugins.Keys)
+				foreach (Note note in attached_plugins.Keys as
+						System.Collections.Generic.IEnumerable<Note>)
 					AttachPlugin (type, note);
 			}
 		}
@@ -559,7 +617,10 @@ namespace Tomboy
 		{
 			if (typeof (NotePlugin).IsAssignableFrom (type)) {
 				foreach (KeyValuePair<Note, PluginTable> pair 
-						in attached_plugins) {
+						in attached_plugins as
+							System.Collections.Generic.IEnumerable<
+								System.Collections.Generic.KeyValuePair<
+									Note, PluginTable>>) {
 					NotePlugin plugin = null;
 
 					if (pair.Value.TryGetValue (type, out plugin)) {
@@ -591,7 +652,8 @@ namespace Tomboy
 
 			// Add the plugin to the list
 			// and load the added plugin for all existing plugged in notes
-			foreach (Type type in asm_plugins) {
+			foreach (Type type in asm_plugins as
+					System.Collections.Generic.IEnumerable<Type>) {
 				if (IsPluginEnabled (type)) {
 					plugin_types.Add (type);
 					AttachPlugin (type);
@@ -609,7 +671,8 @@ namespace Tomboy
 			List<Type> kill_list = new List<Type> ();
 
 			// Find the plugins in the deleted assembly
-			foreach (Type type in plugin_types) {
+			foreach (Type type in plugin_types as
+					System.Collections.Generic.IEnumerable<Type>) {
 				if (type.Assembly.Location == args.FullPath)
 					kill_list.Add (type);
 			}
@@ -628,7 +691,8 @@ namespace Tomboy
 			{
 				List<Type> plugins = new List<Type> (plugin_types.Count);
 
-				foreach (Type type in plugin_types) {
+				foreach (Type type in plugin_types as
+						System.Collections.Generic.IEnumerable<Type>) {
 					if (!IsBuiltin (type))
 						plugins.Add (type);
 				}
@@ -768,7 +832,8 @@ namespace Tomboy
 				
 				try {
 					IList<Type> asm_plugins = FindPluginTypesInFile (file);
-					foreach (Type type in asm_plugins) {
+					foreach (Type type in asm_plugins as
+							System.Collections.Generic.IEnumerable<Type>) {
 						dir_plugin_types.Add (type);
 					}
 				} catch (Exception e) {
