@@ -10,6 +10,7 @@ namespace Tomboy
 		static NoteManager manager;
 		static TomboyTrayIcon tray_icon;
 		static bool tray_icon_showing = false;
+		static bool is_panel_applet = false;
 #if ENABLE_DBUS
 		static RemoteControl remote_control;
 #endif
@@ -51,6 +52,7 @@ namespace Tomboy
 
 			if (cmd_line.UsePanelApplet) {
 				tray_icon_showing = true;
+				is_panel_applet = true;
 
 				// Show the Close item and hide the Quit item
 				am ["CloseWindowAction"].Visible = true;
@@ -131,6 +133,15 @@ namespace Tomboy
 				if (remote_control != null) {
 					Logger.Log ("Tomboy remote control active.");
 				} else {
+					// If Tomboy is already running, open the search window
+					// so the user gets some sort of feedback when they
+					// attempt to run Tomboy again.
+					RemoteControl remote = null;
+					try {
+						remote = RemoteControlProxy.GetInstance ();
+						remote.DisplaySearch ();
+					} catch {}
+
 					Logger.Log ("Tomboy is already running.  Exiting...");
 					System.Environment.Exit (-1);
 				}
@@ -175,6 +186,9 @@ namespace Tomboy
 		
 		static void OnQuitTomboyAction (object sender, EventArgs args)
 		{
+			if (Tomboy.IsPanelApplet)
+				return; // Ignore the quit action
+
 			Logger.Log ("Quitting Tomboy.  Ciao!");
 			Exit (0);
 		}
@@ -241,6 +255,11 @@ namespace Tomboy
 			get { return tray_icon_showing; }
 		}
 		
+		public static bool IsPanelApplet
+		{
+			get { return is_panel_applet; }
+		}
+		
 		public static TomboyTray Tray
 		{
 			get { return tray_icon.TomboyTray; }
@@ -252,6 +271,7 @@ namespace Tomboy
 		bool new_note;
 		bool panel_applet;
 		string new_note_name;
+		bool open_start_here;
 		string open_note_uri;
 		string open_note_name;
 		string highlight_search;
@@ -276,7 +296,8 @@ namespace Tomboy
 				return new_note || 
 						open_note_name != null ||
 						open_note_uri != null || 
-						open_search;
+						open_search ||
+						open_start_here;
 			}
 		}
 
@@ -385,7 +406,7 @@ namespace Tomboy
 
 				case "--start-here":
 					// Open the Start Here note
-					open_note_name = Catalog.GetString ("Start Here");
+					open_start_here = true;
 					break;
 
 				case "--highlight-search":
@@ -509,9 +530,12 @@ namespace Tomboy
 					remote.DisplayNote (new_uri);
 			}
 
+			if (open_start_here)
+				open_note_uri = remote.FindStartHereNote ();
+
 			if (open_note_name != null)
 				open_note_uri = remote.FindNote (open_note_name);
-
+			
 			if (open_note_uri != null) {
 				if (highlight_search != null)
 					remote.DisplayNoteWithSearch (open_note_uri, 
